@@ -1,4 +1,6 @@
-# Copyright (c) 2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2017 Hugo <hugovk@users.noreply.github.com>
+# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
@@ -11,14 +13,15 @@ import six
 import astroid
 
 
-PY33 = sys.version_info >= (3, 3)
+PY34 = sys.version_info >= (3, 4)
 PY36 = sys.version_info >= (3, 6)
+PY33 = sys.version_info >= (3, 3)
 
 
 def _subprocess_transform():
     if six.PY3:
-        communicate = (bytes('string', 'ascii'), bytes('string', 'ascii'))
-        communicate_signature = 'def communicate(self, input=None, timeout=None)'
+        communicate = (bytes("string", "ascii"), bytes("string", "ascii"))
+        communicate_signature = "def communicate(self, input=None, timeout=None)"
         if PY36:
             init = """
             def __init__(self, args, bufsize=0, executable=None,
@@ -41,8 +44,8 @@ def _subprocess_transform():
                 pass
             """
     else:
-        communicate = ('string', 'string')
-        communicate_signature = 'def communicate(self, input=None)'
+        communicate = ("string", "string")
+        communicate_signature = "def communicate(self, input=None)"
         init = """
         def __init__(self, args, bufsize=0, executable=None,
                      stdin=None, stdout=None, stderr=None,
@@ -51,21 +54,26 @@ def _subprocess_transform():
                      startupinfo=None, creationflags=0):
             pass
         """
-    if PY33:
-        wait_signature = 'def wait(self, timeout=None)'
+    if PY34:
+        wait_signature = "def wait(self, timeout=None)"
     else:
-        wait_signature = 'def wait(self)'
+        wait_signature = "def wait(self)"
     if six.PY3:
-        ctx_manager = '''
+        ctx_manager = """
         def __enter__(self): return self
         def __exit__(self, *args): pass
-        '''
+        """
     else:
-        ctx_manager = ''
-    code = textwrap.dedent('''
+        ctx_manager = ""
+    py3_args = ""
+    if PY33:
+        py3_args = "args = []"
+    code = textwrap.dedent(
+        """
     class Popen(object):
         returncode = pid = 0
         stdin = stdout = stderr = file()
+        %(py3_args)s
 
         %(communicate_signature)s:
             return %(communicate)r
@@ -80,15 +88,20 @@ def _subprocess_transform():
         def kill(self):
             pass
         %(ctx_manager)s
-       ''' % {'communicate': communicate,
-              'communicate_signature': communicate_signature,
-              'wait_signature': wait_signature,
-              'ctx_manager': ctx_manager})
+       """
+        % {
+            "communicate": communicate,
+            "communicate_signature": communicate_signature,
+            "wait_signature": wait_signature,
+            "ctx_manager": ctx_manager,
+            "py3_args": py3_args,
+        }
+    )
 
     init_lines = textwrap.dedent(init).splitlines()
-    indented_init = '\n'.join([' ' * 4 + line for line in init_lines])
+    indented_init = "\n".join(" " * 4 + line for line in init_lines)
     code += indented_init
     return astroid.parse(code)
 
 
-astroid.register_module_extender(astroid.MANAGER, 'subprocess', _subprocess_transform)
+astroid.register_module_extender(astroid.MANAGER, "subprocess", _subprocess_transform)
