@@ -24,6 +24,8 @@ import collections
 import operator as operator_mod
 import sys
 
+import itertools
+
 from astroid import Store
 from astroid import arguments
 from astroid import bases
@@ -35,51 +37,53 @@ from astroid import helpers
 from astroid import nodes
 from astroid import util
 
-raw_building = util.lazy_import('raw_building')
-objects = util.lazy_import('objects')
+raw_building = util.lazy_import("raw_building")
+objects = util.lazy_import("objects")
+
 
 def _reflected_name(name):
     return "__r" + name[2:]
 
+
 def _augmented_name(name):
     return "__i" + name[2:]
 
 
-_CONTEXTLIB_MGR = 'contextlib.contextmanager'
-BIN_OP_METHOD = {'+':  '__add__',
-                 '-':  '__sub__',
-                 '/':  '__truediv__',
-                 '//': '__floordiv__',
-                 '*':  '__mul__',
-                 '**': '__pow__',
-                 '%':  '__mod__',
-                 '&':  '__and__',
-                 '|':  '__or__',
-                 '^':  '__xor__',
-                 '<<': '__lshift__',
-                 '>>': '__rshift__',
-                 '@': '__matmul__'
-                }
+_CONTEXTLIB_MGR = "contextlib.contextmanager"
+BIN_OP_METHOD = {
+    "+": "__add__",
+    "-": "__sub__",
+    "/": "__truediv__",
+    "//": "__floordiv__",
+    "*": "__mul__",
+    "**": "__pow__",
+    "%": "__mod__",
+    "&": "__and__",
+    "|": "__or__",
+    "^": "__xor__",
+    "<<": "__lshift__",
+    ">>": "__rshift__",
+    "@": "__matmul__",
+}
 
 REFLECTED_BIN_OP_METHOD = {
-    key: _reflected_name(value)
-    for (key, value) in BIN_OP_METHOD.items()
+    key: _reflected_name(value) for (key, value) in BIN_OP_METHOD.items()
 }
 AUGMENTED_OP_METHOD = {
-    key + "=": _augmented_name(value)
-    for (key, value) in BIN_OP_METHOD.items()
+    key + "=": _augmented_name(value) for (key, value) in BIN_OP_METHOD.items()
 }
 
-UNARY_OP_METHOD = {'+': '__pos__',
-                   '-': '__neg__',
-                   '~': '__invert__',
-                   'not': None, # XXX not '__nonzero__'
-                  }
+UNARY_OP_METHOD = {
+    "+": "__pos__",
+    "-": "__neg__",
+    "~": "__invert__",
+    "not": None,  # XXX not '__nonzero__'
+}
 _UNARY_OPERATORS = {
-    '+': operator_mod.pos,
-    '-': operator_mod.neg,
-    '~': operator_mod.invert,
-    'not': operator_mod.not_,
+    "+": operator_mod.pos,
+    "-": operator_mod.neg,
+    "~": operator_mod.invert,
+    "not": operator_mod.not_,
 }
 
 
@@ -88,6 +92,7 @@ def _infer_unary_op(obj, op):
     value = func(obj)
     return nodes.const_factory(value)
 
+
 nodes.Tuple.infer_unary_op = lambda self, op: _infer_unary_op(tuple(self.elts), op)
 nodes.List.infer_unary_op = lambda self, op: _infer_unary_op(self.elts, op)
 nodes.Set.infer_unary_op = lambda self, op: _infer_unary_op(set(self.elts), op)
@@ -96,25 +101,26 @@ nodes.Dict.infer_unary_op = lambda self, op: _infer_unary_op(dict(self.items), o
 
 # Binary operations
 
-BIN_OP_IMPL = {'+':  lambda a, b: a + b,
-               '-':  lambda a, b: a - b,
-               '/':  lambda a, b: a / b,
-               '//': lambda a, b: a // b,
-               '*':  lambda a, b: a * b,
-               '**': lambda a, b: a ** b,
-               '%':  lambda a, b: a % b,
-               '&':  lambda a, b: a & b,
-               '|':  lambda a, b: a | b,
-               '^':  lambda a, b: a ^ b,
-               '<<': lambda a, b: a << b,
-               '>>': lambda a, b: a >> b,
-              }
+BIN_OP_IMPL = {
+    "+": lambda a, b: a + b,
+    "-": lambda a, b: a - b,
+    "/": lambda a, b: a / b,
+    "//": lambda a, b: a // b,
+    "*": lambda a, b: a * b,
+    "**": lambda a, b: a ** b,
+    "%": lambda a, b: a % b,
+    "&": lambda a, b: a & b,
+    "|": lambda a, b: a | b,
+    "^": lambda a, b: a ^ b,
+    "<<": lambda a, b: a << b,
+    ">>": lambda a, b: a >> b,
+}
 if sys.version_info >= (3, 5):
     # MatMult is available since Python 3.5+.
-    BIN_OP_IMPL['@'] = operator_mod.matmul
+    BIN_OP_IMPL["@"] = operator_mod.matmul
 
 for _KEY, _IMPL in list(BIN_OP_IMPL.items()):
-    BIN_OP_IMPL[_KEY + '='] = _IMPL
+    BIN_OP_IMPL[_KEY + "="] = _IMPL
 
 
 @decorators.yes_if_nothing_inferred
@@ -128,29 +134,28 @@ def const_infer_binary_op(self, opnode, operator, other, context, _):
             except TypeError:
                 # ArithmeticError is not enough: float >> float is a TypeError
                 yield not_implemented
-            except Exception: # pylint: disable=broad-except
+            except Exception:  # pylint: disable=broad-except
                 yield util.Uninferable
         except TypeError:
             yield not_implemented
-    elif isinstance(self.value, str) and operator == '%':
+    elif isinstance(self.value, str) and operator == "%":
         # TODO(cpopa): implement string interpolation later on.
         yield util.Uninferable
     else:
         yield not_implemented
 
+
 nodes.Const.infer_binary_op = const_infer_binary_op
 
 
 def _multiply_seq_by_int(self, opnode, other, context):
     node = self.__class__(parent=opnode)
-    elts = []
-    filtered_elts = (elt for elt in self.elts if elt is not util.Uninferable)
-    for elt in filtered_elts:
-        inferred = helpers.safe_infer(elt, context)
-        if inferred is None:
-            inferred = util.Uninferable
-        elts.append(inferred)
-    node.elts = elts * other.value
+    filtered_elts = (
+        helpers.safe_infer(elt, context) or util.Uninferable
+        for elt in self.elts
+        if elt is not util.Uninferable
+    )
+    node.elts = list(filtered_elts) * other.value
     return node
 
 
@@ -169,18 +174,21 @@ def _filter_uninferable_nodes(elts, context):
 @decorators.yes_if_nothing_inferred
 def tl_infer_binary_op(self, opnode, operator, other, context, method):
     not_implemented = nodes.Const(NotImplemented)
-    if isinstance(other, self.__class__) and operator == '+':
+    if isinstance(other, self.__class__) and operator == "+":
         node = self.__class__(parent=opnode)
-        elts = list(_filter_uninferable_nodes(self.elts, context))
-        elts += list(_filter_uninferable_nodes(other.elts, context))
-        node.elts = elts
+        node.elts = list(
+            itertools.chain(
+                _filter_uninferable_nodes(self.elts, context),
+                _filter_uninferable_nodes(other.elts, context),
+            )
+        )
         yield node
-    elif isinstance(other, nodes.Const) and operator == '*':
+    elif isinstance(other, nodes.Const) and operator == "*":
         if not isinstance(other.value, int):
             yield not_implemented
             return
         yield _multiply_seq_by_int(self, opnode, other, context)
-    elif isinstance(other, bases.Instance) and operator == '*':
+    elif isinstance(other, bases.Instance) and operator == "*":
         # Verify if the instance supports __index__.
         as_index = helpers.class_instance_as_index(other)
         if not as_index:
@@ -190,6 +198,7 @@ def tl_infer_binary_op(self, opnode, operator, other, context, method):
     else:
         yield not_implemented
 
+
 nodes.Tuple.infer_binary_op = tl_infer_binary_op
 nodes.List.infer_binary_op = tl_infer_binary_op
 
@@ -198,6 +207,7 @@ nodes.List.infer_binary_op = tl_infer_binary_op
 def instance_class_infer_binary_op(self, opnode, operator, other, context, method):
     return method.infer_call_result(self, context)
 
+
 bases.Instance.infer_binary_op = instance_class_infer_binary_op
 nodes.ClassDef.infer_binary_op = instance_class_infer_binary_op
 
@@ -207,37 +217,39 @@ nodes.ClassDef.infer_binary_op = instance_class_infer_binary_op
 """the assigned_stmts method is responsible to return the assigned statement
 (e.g. not inferred) according to the assignment type.
 
-The `asspath` argument is used to record the lhs path of the original node.
-For instance if we want assigned statements for 'c' in 'a, (b,c)', asspath
+The `assign_path` argument is used to record the lhs path of the original node.
+For instance if we want assigned statements for 'c' in 'a, (b,c)', assign_path
 will be [1, 1] once arrived to the Assign node.
 
 The `context` argument is the current inference context which should be given
 to any intermediary inference necessary.
 """
 
-def _resolve_looppart(parts, asspath, context):
+
+def _resolve_looppart(parts, assign_path, context):
     """recursive function to resolve multiple assignments on loops"""
-    asspath = asspath[:]
-    index = asspath.pop(0)
+    assign_path = assign_path[:]
+    index = assign_path.pop(0)
     for part in parts:
         if part is util.Uninferable:
             continue
-        # XXX handle __iter__ and log potentially detected errors
-        if not hasattr(part, 'itered'):
+        if not hasattr(part, "itered"):
             continue
         try:
             itered = part.itered()
         except TypeError:
-            continue # XXX log error
+            continue
         for stmt in itered:
             index_node = nodes.Const(index)
             try:
                 assigned = stmt.getitem(index_node, context)
-            except (AttributeError,
-                    exceptions.AstroidTypeError,
-                    exceptions.AstroidIndexError):
+            except (
+                AttributeError,
+                exceptions.AstroidTypeError,
+                exceptions.AstroidIndexError,
+            ):
                 continue
-            if not asspath:
+            if not assign_path:
                 # we achieved to resolved the assignment path,
                 # don't infer the last part
                 yield assigned
@@ -247,48 +259,58 @@ def _resolve_looppart(parts, asspath, context):
                 # we are not yet on the last part of the path
                 # search on each possibly inferred value
                 try:
-                    yield from _resolve_looppart(assigned.infer(context), asspath, context)
+                    yield from _resolve_looppart(
+                        assigned.infer(context), assign_path, context
+                    )
                 except exceptions.InferenceError:
                     break
 
+
 @decorators.raise_if_nothing_inferred
-def for_assigned_stmts(self, node=None, context=None, asspath=None):
-    if isinstance(self, nodes.AsyncFor) or getattr(self, 'is_async', False):
+def for_assigned_stmts(self, node=None, context=None, assign_path=None):
+    if isinstance(self, nodes.AsyncFor) or getattr(self, "is_async", False):
         # Skip inferring of async code for now
-        return dict(node=self, unknown=node,
-                    assign_path=asspath, context=context)
-    if asspath is None:
+        return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    if assign_path is None:
         for lst in self.iter.infer(context):
             if isinstance(lst, (nodes.Tuple, nodes.List)):
                 yield from lst.elts
     else:
-        yield from _resolve_looppart(self.iter.infer(context), asspath, context)
-    return dict(node=self, unknown=node,
-                assign_path=asspath, context=context)
+        yield from _resolve_looppart(self.iter.infer(context), assign_path, context)
+    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+
 
 nodes.For.assigned_stmts = for_assigned_stmts
 nodes.Comprehension.assigned_stmts = for_assigned_stmts
 
 
-def sequence_assigned_stmts(self, node=None, context=None, asspath=None):
-    if asspath is None:
-        asspath = []
+def sequence_assigned_stmts(self, node=None, context=None, assign_path=None):
+    if assign_path is None:
+        assign_path = []
     try:
         index = self.elts.index(node)
     except ValueError as exc:
         raise exceptions.InferenceError(
-            'Tried to retrieve a node {node!r} which does not exist',
-            node=self, assign_path=asspath, context=context) from exc
+            "Tried to retrieve a node {node!r} which does not exist",
+            node=self,
+            assign_path=assign_path,
+            context=context,
+        ) from exc
+
+    assign_path.insert(0, index)
+    return self.parent.assigned_stmts(
+        node=self, context=context, assign_path=assign_path
+    )
 
-    asspath.insert(0, index)
-    return self.parent.assigned_stmts(node=self, context=context, asspath=asspath)
 
 nodes.Tuple.assigned_stmts = sequence_assigned_stmts
 nodes.List.assigned_stmts = sequence_assigned_stmts
 
 
-def assend_assigned_stmts(self, node=None, context=None, asspath=None):
+def assend_assigned_stmts(self, node=None, context=None, assign_path=None):
     return self.parent.assigned_stmts(node=self, context=context)
+
+
 nodes.AssignName.assigned_stmts = assend_assigned_stmts
 nodes.AssignAttr.assigned_stmts = assend_assigned_stmts
 
@@ -300,24 +322,22 @@ def _arguments_infer_argname(self, name, context):
         yield util.Uninferable
         return
     # first argument of instance/class method
-    if self.args and getattr(self.args[0], 'name', None) == name:
+    if self.args and getattr(self.args[0], "name", None) == name:
         functype = self.parent.type
         cls = self.parent.parent.scope()
-        is_metaclass = isinstance(cls, nodes.ClassDef) and cls.type == 'metaclass'
+        is_metaclass = isinstance(cls, nodes.ClassDef) and cls.type == "metaclass"
         # If this is a metaclass, then the first argument will always
         # be the class, not an instance.
-        if is_metaclass or functype == 'classmethod':
+        if is_metaclass or functype == "classmethod":
             yield cls
             return
-        if functype == 'method':
-            yield bases.Instance(self.parent.parent.frame())
+        if functype == "method":
+            yield bases.Instance(cls)
             return
 
     if context and context.callcontext:
-        call_site = arguments.CallSite(context.callcontext,
-                                       context.extra_context)
-        for value in call_site.infer_argument(self.parent, name, context):
-            yield value
+        call_site = arguments.CallSite(context.callcontext, context.extra_context)
+        yield from call_site.infer_argument(self.parent, name, context)
         return
 
     if name == self.vararg:
@@ -340,7 +360,7 @@ def _arguments_infer_argname(self, name, context):
         yield util.Uninferable
 
 
-def arguments_assigned_stmts(self, node=None, context=None, asspath=None):
+def arguments_assigned_stmts(self, node=None, context=None, assign_path=None):
     if context.callcontext:
         # reset call context/name
         callcontext = context.callcontext
@@ -350,36 +370,39 @@ def arguments_assigned_stmts(self, node=None, context=None, asspath=None):
         return args.infer_argument(self.parent, node.name, context)
     return _arguments_infer_argname(self, node.name, context)
 
+
 nodes.Arguments.assigned_stmts = arguments_assigned_stmts
 
 
 @decorators.raise_if_nothing_inferred
-def assign_assigned_stmts(self, node=None, context=None, asspath=None):
-    if not asspath:
+def assign_assigned_stmts(self, node=None, context=None, assign_path=None):
+    if not assign_path:
         yield self.value
         return None
-    yield from _resolve_asspart(self.value.infer(context), asspath, context)
+    yield from _resolve_assignment_parts(
+        self.value.infer(context), assign_path, context
+    )
 
-    return dict(node=self, unknown=node,
-                assign_path=asspath, context=context)
+    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
 
 
-def assign_annassigned_stmts(self, node=None, context=None, asspath=None):
-    for inferred in assign_assigned_stmts(self, node, context, asspath):
+def assign_annassigned_stmts(self, node=None, context=None, assign_path=None):
+    for inferred in assign_assigned_stmts(self, node, context, assign_path):
         if inferred is None:
             yield util.Uninferable
         else:
             yield inferred
 
+
 nodes.Assign.assigned_stmts = assign_assigned_stmts
 nodes.AnnAssign.assigned_stmts = assign_annassigned_stmts
 nodes.AugAssign.assigned_stmts = assign_assigned_stmts
 
 
-def _resolve_asspart(parts, asspath, context):
+def _resolve_assignment_parts(parts, assign_path, context):
     """recursive function to resolve multiple assignments"""
-    asspath = asspath[:]
-    index = asspath.pop(0)
+    assign_path = assign_path[:]
+    index = assign_path.pop(0)
     for part in parts:
         assigned = None
         if isinstance(part, nodes.Dict):
@@ -389,19 +412,17 @@ def _resolve_asspart(parts, asspath, context):
             except IndexError:
                 return
 
-        elif hasattr(part, 'getitem'):
+        elif hasattr(part, "getitem"):
             index_node = nodes.Const(index)
             try:
                 assigned = part.getitem(index_node, context)
-            # XXX raise a specific exception to avoid potential hiding of
-            # unexpected exception ?
             except (exceptions.AstroidTypeError, exceptions.AstroidIndexError):
                 return
 
         if not assigned:
             return
 
-        if not asspath:
+        if not assign_path:
             # we achieved to resolved the assignment path, don't infer the
             # last part
             yield assigned
@@ -411,20 +432,21 @@ def _resolve_asspart(parts, asspath, context):
             # we are not yet on the last part of the path search on each
             # possibly inferred value
             try:
-                yield from _resolve_asspart(assigned.infer(context), asspath, context)
+                yield from _resolve_assignment_parts(
+                    assigned.infer(context), assign_path, context
+                )
             except exceptions.InferenceError:
                 return
 
 
 @decorators.raise_if_nothing_inferred
-def excepthandler_assigned_stmts(self, node=None, context=None, asspath=None):
+def excepthandler_assigned_stmts(self, node=None, context=None, assign_path=None):
     for assigned in node_classes.unpack_infer(self.type):
         if isinstance(assigned, nodes.ClassDef):
             assigned = objects.ExceptionInstance(assigned)
 
         yield assigned
-    return dict(node=self, unknown=node,
-                assign_path=asspath, context=context)
+    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
 
 
 nodes.ExceptHandler.assigned_stmts = excepthandler_assigned_stmts
@@ -457,9 +479,9 @@ def _infer_context_manager(self, mgr, context):
 
         possible_yield_points = func.nodes_of_class(nodes.Yield)
         # Ignore yields in nested functions
-        yield_point = next((node for node in possible_yield_points
-                            if node.scope() == func),
-                           None)
+        yield_point = next(
+            (node for node in possible_yield_points if node.scope() == func), None
+        )
         if yield_point:
             if not yield_point.value:
                 const = nodes.Const(None)
@@ -470,8 +492,12 @@ def _infer_context_manager(self, mgr, context):
                 yield from yield_point.value.infer(context=context)
     elif isinstance(inferred, bases.Instance):
         try:
-            enter = next(inferred.igetattr('__enter__', context=context))
-        except (StopIteration, exceptions.InferenceError, exceptions.AttributeInferenceError):
+            enter = next(inferred.igetattr("__enter__", context=context))
+        except (
+            StopIteration,
+            exceptions.InferenceError,
+            exceptions.AttributeInferenceError,
+        ):
             return
         if not isinstance(enter, bases.BoundMethod):
             return
@@ -481,7 +507,7 @@ def _infer_context_manager(self, mgr, context):
 
 
 @decorators.raise_if_nothing_inferred
-def with_assigned_stmts(self, node=None, context=None, asspath=None):
+def with_assigned_stmts(self, node=None, context=None, assign_path=None):
     """Infer names and other nodes from a *with* statement.
 
     This enables only inference for name binding in a *with* statement.
@@ -503,60 +529,76 @@ def with_assigned_stmts(self, node=None, context=None, asspath=None):
     Arguments:
         self: nodes.With
         node: The target of the assignment, `as (a, b)` in `with foo as (a, b)`.
-        context: TODO
-        asspath: TODO
+        context: Inference context used for caching already inferred objects
+        assign_path:
+            A list of indices, where each index specifies what item to fetch from
+            the inference results.
     """
     try:
         mgr = next(mgr for (mgr, vars) in self.items if vars == node)
     except StopIteration:
         return None
-    if asspath is None:
-        for result in _infer_context_manager(self, mgr, context):
-            yield result
+    if assign_path is None:
+        yield from _infer_context_manager(self, mgr, context)
     else:
         for result in _infer_context_manager(self, mgr, context):
-            # Walk the asspath and get the item at the final index.
+            # Walk the assign_path and get the item at the final index.
             obj = result
-            for index in asspath:
-                if not hasattr(obj, 'elts'):
+            for index in assign_path:
+                if not hasattr(obj, "elts"):
                     raise exceptions.InferenceError(
-                        'Wrong type ({targets!r}) for {node!r} assignment',
-                        node=self, targets=node, assign_path=asspath,
-                        context=context)
+                        "Wrong type ({targets!r}) for {node!r} assignment",
+                        node=self,
+                        targets=node,
+                        assign_path=assign_path,
+                        context=context,
+                    )
                 try:
                     obj = obj.elts[index]
                 except IndexError as exc:
                     raise exceptions.InferenceError(
-                        'Tried to infer a nonexistent target with index {index} '
-                        'in {node!r}.', node=self, targets=node,
-                        assign_path=asspath, context=context) from exc
+                        "Tried to infer a nonexistent target with index {index} "
+                        "in {node!r}.",
+                        node=self,
+                        targets=node,
+                        assign_path=assign_path,
+                        context=context,
+                    ) from exc
                 except TypeError as exc:
                     raise exceptions.InferenceError(
-                        'Tried to unpack a non-iterable value '
-                        'in {node!r}.', node=self, targets=node,
-                        assign_path=asspath, context=context) from exc
+                        "Tried to unpack a non-iterable value " "in {node!r}.",
+                        node=self,
+                        targets=node,
+                        assign_path=assign_path,
+                        context=context,
+                    ) from exc
             yield obj
-    return dict(node=self, unknown=node,
-                assign_path=asspath, context=context)
+    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+
 
 nodes.With.assigned_stmts = with_assigned_stmts
 
 
 @decorators.yes_if_nothing_inferred
-def starred_assigned_stmts(self, node=None, context=None, asspath=None):
+def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
     """
     Arguments:
         self: nodes.Starred
-        node: TODO
-        context: TODO
-        asspath: TODO
+        node: a node related to the current underlying Node.
+        context: Inference context used for caching already inferred objects
+        assign_path:
+            A list of indices, where each index specifies what item to fetch from
+            the inference results.
     """
     # pylint: disable=too-many-locals,too-many-branches,too-many-statements
     def _determine_starred_iteration_lookups(starred, target, lookups):
         # Determine the lookups for the rhs of the iteration
         itered = target.itered()
         for index, element in enumerate(itered):
-            if isinstance(element, nodes.Starred) and element.value.name == starred.value.name:
+            if (
+                isinstance(element, nodes.Starred)
+                and element.value.name == starred.value.name
+            ):
                 lookups.append((index, len(itered)))
                 break
             if isinstance(element, nodes.Tuple):
@@ -565,10 +607,13 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
 
     stmt = self.statement()
     if not isinstance(stmt, (nodes.Assign, nodes.For)):
-        raise exceptions.InferenceError('Statement {stmt!r} enclosing {node!r} '
-                                        'must be an Assign or For node.',
-                                        node=self, stmt=stmt, unknown=node,
-                                        context=context)
+        raise exceptions.InferenceError(
+            "Statement {stmt!r} enclosing {node!r} " "must be an Assign or For node.",
+            node=self,
+            stmt=stmt,
+            unknown=node,
+            context=context,
+        )
 
     if context is None:
         context = contextmod.InferenceContext()
@@ -578,17 +623,20 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
         lhs = stmt.targets[0]
 
         if sum(1 for _ in lhs.nodes_of_class(nodes.Starred)) > 1:
-            raise exceptions.InferenceError('Too many starred arguments in the '
-                                            ' assignment targets {lhs!r}.',
-                                            node=self, targets=lhs,
-                                            unknown=node, context=context)
+            raise exceptions.InferenceError(
+                "Too many starred arguments in the " " assignment targets {lhs!r}.",
+                node=self,
+                targets=lhs,
+                unknown=node,
+                context=context,
+            )
 
         try:
             rhs = next(value.infer(context))
         except exceptions.InferenceError:
             yield util.Uninferable
             return
-        if rhs is util.Uninferable or not hasattr(rhs, 'itered'):
+        if rhs is util.Uninferable or not hasattr(rhs, "itered"):
             yield util.Uninferable
             return
 
@@ -620,10 +668,7 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
                     continue
                 # We're done
                 packed = nodes.List(
-                    ctx=Store,
-                    parent=self,
-                    lineno=lhs.lineno,
-                    col_offset=lhs.col_offset,
+                    ctx=Store, parent=self, lineno=lhs.lineno, col_offset=lhs.col_offset
                 )
                 packed.postinit(elts=elts)
                 yield packed
@@ -635,7 +680,9 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
         except exceptions.InferenceError:
             yield util.Uninferable
             return
-        if inferred_iterable is util.Uninferable or not hasattr(inferred_iterable, 'itered'):
+        if inferred_iterable is util.Uninferable or not hasattr(
+            inferred_iterable, "itered"
+        ):
             yield util.Uninferable
             return
         try:
@@ -648,7 +695,7 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
 
         if not isinstance(target, nodes.Tuple):
             raise exceptions.InferenceError(
-                'Could not make sense of this, the target must be a tuple',
+                "Could not make sense of this, the target must be a tuple",
                 context=context,
             )
 
@@ -656,8 +703,7 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
         _determine_starred_iteration_lookups(self, target, lookups)
         if not lookups:
             raise exceptions.InferenceError(
-                'Could not make sense of this, needs at least a lookup',
-                context=context,
+                "Could not make sense of this, needs at least a lookup", context=context
             )
 
         # Make the last lookup a slice, since that what we want for a Starred node
@@ -666,7 +712,7 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
 
         lookup_slice = slice(
             last_element_index,
-            None if is_starred_last else (last_element_length - last_element_index)
+            None if is_starred_last else (last_element_length - last_element_index),
         )
         lookups[-1] = lookup_slice
 
@@ -684,7 +730,7 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
 
             found_element = None
             for lookup in lookups:
-                if not hasattr(element, 'itered'):
+                if not hasattr(element, "itered"):
                     break
                 if not isinstance(lookup, slice):
                     # Grab just the index, not the whole length
@@ -702,10 +748,7 @@ def starred_assigned_stmts(self, node=None, context=None, asspath=None):
                     found_element = element
 
             unpacked = nodes.List(
-                ctx=Store,
-                parent=self,
-                lineno=self.lineno,
-                col_offset=self.col_offset,
+                ctx=Store, parent=self, lineno=self.lineno, col_offset=self.col_offset
             )
             unpacked.postinit(elts=found_element or [])
             yield unpacked
